# This file demonstrates O(1) sorting.
# It assumes the array to be sorted has first been encoded in a convenient
# format. The output is given in the same format. Encoding and decoding are
# handled by the function "sorting". The actual sorting is handled by
# "fast_sorting". "fast_sorting" runs in O(1) in the desired computational
# model.
 
from math import *

def sorting(array):
  size=len(array)
  if size==0:
    return []
  minimum=min(array)
  nonnegative_array=[x-minimum for x in array]
  maximum=max(nonnegative_array)
  width=int(log(max(maximum,1))/log(2))+1
  encoded_array=encode(nonnegative_array,width)
  encoded_sorted_array=fast_sorting(encoded_array,size,width)
  sorted_array=decode(encoded_sorted_array,size,width)
  return [x+minimum for x in sorted_array]

def encode(array,width):
  encoded_array=0
  for i in range(len(array)):
    encoded_array+=array[i]<<(width*i)
  return encoded_array

def decode(array,size,width):
  decoded_array=[0]*size
  for i in range(size):
    decoded_array[i]=array & mask(width)
    array>>=width
  return decoded_array


def fast_sorting(array,size,width):
  new_width=width+size+1
  # +size is for uniqueness, +1 is to have a carry bit.
  wide_array=change_width(array,size,width,new_width)
  uniq_array=(wide_array<<size)+counting(size,new_width)
  uniq_array=uniq_sorting(uniq_array,size,new_width)
  uniq_array&=ones(size,new_width)*(mask(width+size)-mask(size))
  wide_array=uniq_array>>size
  return change_width(wide_array,size,new_width,width)

def widen(array,size,width,new_width):
  # This assumes new_width>=(size+1)*width
  wide_array=array*ones(size,new_width-width)
  return wide_array & (ones(size,new_width)*mask(width))

def narrow(array,size,width,new_width):
  # This assumes width>=new_width*size
  # a more generic 'narrow' would mask out the now-obsolete higher bits
  # before continuing.
  return array % mask(width-new_width)

def change_width(array,size,width,new_width):
  temp_width=(size+1)*max(width,new_width)
  wide_array=widen(array,size,width,temp_width)
  return narrow(wide_array,size,temp_width,new_width)

def uniq_sorting(array,size,width):
  # assumes uniqueness
  # assumes width is large enough to hold a carry bit.
  batch_width=width*size
  repeated_array=array*ones(size,batch_width)
  wide_array=widen(array,size,width,batch_width)*ones(size,width)
  # Even though the parameters for use of 'widen' aren't met here
  # (batch_width being still a tad too small), because we allowed for
  # a carry bit, the results should still be ok.
  pattern=lt(wide_array,repeated_array,size*size,width)
  pattern%=mask(size*width)
  pattern=change_width(pattern,size,width,batch_width)
  pattern*=ones(size,width)
  count=counting(size,width)*ones(size,batch_width)
  pattern=equals(pattern,count,size*size,width)
  return ((pattern*mask(width))&wide_array)%mask(batch_width)

def mask(size):
  return (1<<size)-1

def ones(size,width):
  return mask(size*width)/mask(width)

def counting(size,width):
  return (1<<(size*width))*(size-1)/mask(width)- \
           ((1<<(size*width))-(1<<width))/(mask(width)**2)

def equals(a,b,size,width):
  return ge(a,b,size,width) & ge(b,a,size,width)

def lt(a,b,size,width):
  return ones(size,width) & ~ge(a,b,size,width)

def ge(a,b,size,width):
  return ((a+(~b & datamask(size,width))+ones(size,width))
          & flagmask(size,width)) >> (width-1)

def flagmask(size,width):
  return ones(size,width) << (width-1) 

def datamask(size,width):
  return ones(size,width) * mask(width-1)

